package thermo;

/**
 * <p>Title: JThermo</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: www.LancOS.com</p>
 * @author Claudio Lanconelli
 * @version 1.0
 */

public class I2CBus
{
	private SerialConnection sercon;
	private int delay;
	private int errno;

	private final static int BUSYDELAY = 100;

	public final static int IICERR_BUSBUSY = -11;
	public final static int IICERR_NOTACK = -12;
	public final static int IICERR_NOADDRACK = -13;
	public final static int IICERR_SDACONFLICT = -14;
	public final static int IICERR_STOP = -20;

    public I2CBus()
	{
		try {
			sercon = new SerialConnection();
		}
		catch(NoClassDefFoundError e) {
			e.printStackTrace();
			javax.swing.JOptionPane.showMessageDialog( null,
						"Install Java COMM API or equivalent (RXTX), please",
						"COMM API Classes Not Found",
						javax.swing.JOptionPane.ERROR_MESSAGE);
			System.exit(1);
		};

		delay = 5;  //microseconds
		errno = 0;
	}

	public void setDelay(int value)
	{
		if (value > 0)
		    delay = value;
	}

	public int getDelay()
	{
		return delay;
	}

	public int getErrorNumber()
	{
		return errno;
	}
	public void clearErrorNumber()
	{
		errno = 0;
	}

	private void Wait()
	{
		int j;
		for (j = 0; j < delay; j++)
		    ;
	/**
		// use instead of busy loop on fast java virtual machines!! (may be PentiumX at 100GHz with 128 GByte RAM)
		try {
			Thread.sleep(0, delay*1000);
		} catch (InterruptedException e) {
		    e.printStackTrace();
		}
	**/
	}

	private int CheckBusy()
	{
		int count;

		for (count = BUSYDELAY; count > 0 && isSCLSDA(); count--)
			Wait();
		return count > 0 ? IICERR_BUSBUSY : 0;
	}

	private int SendStart()
	{
		int err;

		setSCLSDA();
		Wait();
		err = CheckBusy();
		if (err != 0)
			return err;
		clearSDA();
		Wait(); 	// tHD;STA = 4 usec
		clearSCL();
		Wait();

		return 0;
	}

	private int SendStop()
	{
		clearSCL();
		Wait();
		clearSDA();
		Wait();
		setSCL();
		Wait();	// tSU;STOP = 4.7 usec
		setSDA();
		Wait();
		if (!getSDA())
		{
		//	UserDebug(UserApp2, "I2CBus::SendStop() *** SDA error\n");
			return IICERR_SDACONFLICT;
		}
		//tBUF = 4.7 usec
		Wait();

		return 0;
	}

	private short SendBitMast(boolean b)
	{
		bitSDA(b);
		Wait();	// tSU;DAT = 250 nsec (tLOW / 2 = 2 usec)
		setSCL();
		Wait();	// tHIGH / 2 = 2 usec
		if (!getSDA() != !b)
			return IICERR_SDACONFLICT;
		Wait();	// tHIGH / 2 = 2 usec
		clearSCL();
		Wait();	// tHD;DATA = 300 nsec (tLOW / 2 = 2 usec)

		return 0;
	}

	// returns a negative number if there's an error, 0 or 1 otherwise
	private boolean RecBitMast()
	{
		boolean b;

		setSDA();		// to receive data SDA must be high
		Wait();	    // tSU;DAT = 250 nsec (tLOW / 2 = 2 usec)
		setSCL();
		Wait(); 	// tHIGH / 2 = 2 usec
		b = getSDA();
		Wait(); 	// tHIGH / 2 = 2 usec
		clearSCL();
		Wait();	// tHD;DATA = 300 nsec (tLOW / 2 = 2 usec)

		return b;
	}

	// OK, ora ci alziamo di un livello: operiamo sul byte
	public int WriteByte(int by)
	{
		int k;

		for (k = 7; k >= 0; k--)
		{
		    boolean b = ( (by & (1<<k)) != 0 );
			int err = SendBitMast(b);
			if (err != 0)
			{
				return err;
			}
		}

		boolean lrb = RecBitMast();	// acknowledge bit
		if (lrb)
			return IICERR_NOTACK;

		return 0;
	}

	public int ReadByte(boolean ack)
	{
		int k;
		short val = 0;

		for (k = 7; k >= 0; k--)
		{
			if ( RecBitMast() )
				val |= (1 << k);
		}
		k = SendBitMast(ack);   //send acknowledge
		if (k != 0)
			return k;
		setSDA();	/* rilascia SDA allo slave trasmitter */

		return val;
	}

	public int Start(int slave)
	{
		int temp;

		// Invio Start
		temp = SendStart();
		if ( temp != 0 )
		{
			errno = temp;
			return errno;
		}

		if ( (temp = WriteByte(slave)) != 0 )
		{
			errno = (temp == IICERR_NOTACK) ? IICERR_NOADDRACK : temp;
			return errno;
		}

		return 0;
	}

	public int Stop()
	{
		errno = SendStop() != 0 ? IICERR_STOP : 0;

		return errno;
	}

	/* se length e` zero, viene ricevuto un byte senza ack per cercare di
	 * liberare il bus da uno slave che per qualche motivo e` ancora in
	 * trasmissione. In questo caso data deve puntare ad un buffer di (almeno)
	 * un byte.
	 */
	public int StartRead(int slave, int data[], int length)
	{
		int temp;
		int j = 0;
		int len = length;

		if (len > 0)
		{
			// send Start
		    temp = SendStart();
			if ( temp != 0 )
			{
				errno = temp;
				return 0;
			}

			if ( (temp = WriteByte(slave | 1)) != 0 )
			{
				errno = (temp == IICERR_NOTACK) ? IICERR_NOADDRACK : temp;
				return 0;
			}

			while (len > 1)
			{
				temp = ReadByte(false);
				if (temp < 0)
				{
					errno = temp;
					return length-len;
				}
				data[j++] = (byte)temp;
				len--;
			}
		}
		// last byte received without acknowledge
		if ((temp = ReadByte(true)) < 0)
		{
			errno = temp;
			return length-len;
		}
		len--;

		data[j] = (byte)temp;
		errno = 0;

		return length-len;
	}

	public int StartWrite(int slave, int data[], int length)
	{
		int error;
		int j = 0;
		int len = length;

		if (len == 0)
			return 0;

		error = SendStart();
		if (error != 0)
		{
			errno = error;
			return 0;
		}
		error = WriteByte(slave & 0xFE);
		if (error != 0)
		{
			errno = (error == IICERR_NOTACK) ? IICERR_NOADDRACK : error;
			return 0;
		}

		while (len > 0)
		{
		    error = WriteByte(data[j++]);
			if (error != 0)
			{
				errno = error;
				return length-len;
			}
			len--;
		}

		return length-len;
	}

	public int Read(int slave, int data[], int length)
	{
		int len;

		len = StartRead(slave, data, length);
		if (len == length)
			if ( errno != 0 || Stop() != 0 )
				len = 0;
		return len;
	}

	public int Write(int slave, int data[], int length)
	{
		int len;

		len = StartWrite(slave, data, length);
		if (len == length)
			if ( errno != 0 || Stop() != 0)
				len = 0;
		return len;
	}

/**
	public void Reset()
	{
	//	Read(0x00, &c, 0);
		setSCLSDA();
		WaitMsec(100);
	}
**/
	public void Open(int port)
	{
		errno = 0;

		try {
			sercon.openConnection(port - 1);
		} catch (SerialConnectionException e) {
			e.printStackTrace();
		}

		setSCLSDA();
		PowerPulse();
	//	rs232.SetSerialBreak(1);
	//	WaitMsec(100);
	//	Reset();
	}

	protected void PowerPulse()
	{
		sercon.sendBreak(50);      //50 msec power pulse
	}
	protected void PowerPulse(int msec)
	{
		sercon.sendBreak(msec);
	}

	public void Close()
	{
		setSCLSDA();
	//	rs232.SetSerialBreak(0);
		sercon.closeConnection();
	}

	private void setSCLSDA()
	{
		sercon.setRTS(true);
		sercon.setDTR(true);
	}

	private void setSDA()
	{
		sercon.setDTR(true);
	}

	private void clearSDA()
	{
		sercon.setDTR(false);
	}

	private void setSCL()
	{
		sercon.setRTS(true);
	}

	private void clearSCL()
	{
		sercon.setRTS(false);
	}

	private boolean isSCLSDA()
	{
		return sercon.getCTS();
	}

	private boolean getSDA()
	{
		return sercon.getCTS();
	}

	private boolean getSCL()
	{
		return true;
	}

	private void bitSDA(boolean bit)
	{
		sercon.setDTR(bit);
	}
}